function Ifilt = nlm(I, searchWindowRadius, averageFilterRadius, sigma, nlm_sigma)
    
    % pad image to reduce boundary artifacts
    I = padarray(I,[averageFilterRadius averageFilterRadius],'symmetric');

    % initialize filtered image as 0
    Ifilt = zeros(size(I));
    
    % smoothing kernel
    kernel = fspecial('gaussian', [2*averageFilterRadius+1 2*averageFilterRadius+1], sigma);
    
    
    for ky= 1+averageFilterRadius:size(I,1)-averageFilterRadius
        
        for kx= 1+averageFilterRadius:size(I,2)-averageFilterRadius
            
            % extract current window
            %currentWindow = I(  ky-averageFilterRadius:ky+averageFilterRadius, ...
            %                    kx-averageFilterRadius:kx+averageFilterRadius, :);
            currentWindow = I(  ky-averageFilterRadius:ky+averageFilterRadius, ...
                                kx-averageFilterRadius:kx+averageFilterRadius);
                            
            % accumulated normalization factor
            % normalizationFactor = zeros([1 1 size(I,3)]);
            normalizationFactor = 0;
            weight_max = 0;
            
            % go over a window of size 2*searchWindowRadius+1 around the
            % current pixel, compute weights based on difference of neighborhoods, 
            % sum the weighted intensity
            
            % hint: do !not! include the neighborhood centered at the
            % current pixel in this loop, it will throw off the weights

            minWindowY = max(ky-searchWindowRadius,1+averageFilterRadius);
            maxWindowY = min(ky+searchWindowRadius,size(I,1)-averageFilterRadius);
            minWindowX = max(kx-searchWindowRadius,1+averageFilterRadius);
            maxWindowX = min(kx+searchWindowRadius,size(I,2)-averageFilterRadius);
            for ly= minWindowY:maxWindowY
                for lx= minWindowX:maxWindowX
                    if (ky~=ly) || (kx~=lx)
                        %neighborWindow = I(  ly-averageFilterRadius:ly+averageFilterRadius, ...
                        %                 lx-averageFilterRadius:lx+averageFilterRadius, :);
                        neighborWindow = I(  ly-averageFilterRadius:ly+averageFilterRadius, ...
                                         lx-averageFilterRadius:lx+averageFilterRadius);
                        diff = neighborWindow - currentWindow;
                        kdiff2 = (kernel.*diff).^2;
                        summ = sum(kdiff2(:));
                        w = exp(-summ/(nlm_sigma^2));
                        %Ifilt(ky,kx,:) = Ifilt(ky,kx,:) + w*I(ly,lx,:);
                        Ifilt(ky,kx) = Ifilt(ky,kx) + w.*I(ly,lx);
                        normalizationFactor = normalizationFactor + w;
                        if w > weight_max
                            weight_max = w;
                        end
                    end
                end
            end
                  
            
%            % this one makes it a bit better - add current pixel as well with max weight computed from all other neighborhoods           
            normalizationFactor = normalizationFactor + weight_max;
            %Ifilt(ky,kx,:) = Ifilt(ky,kx,:) + weight_max .* I(ky,kx,:); 
            Ifilt(ky,kx) = Ifilt(ky,kx) + weight_max .* I(ky,kx); 

            % normalize pixel value
            %Ifilt(ky,kx,:) = Ifilt(ky,kx,:) ./ normalizationFactor;
            Ifilt(ky,kx) = Ifilt(ky,kx) ./ normalizationFactor;

        end
    end
    
    % crop black boundary
    Ifilt = Ifilt(averageFilterRadius+1:end-averageFilterRadius, averageFilterRadius+1:end-averageFilterRadius);
end
